(require 'cl)
;;;
;;; -------------------------------------------------------------------------
;;; generate a directing binding for a key. Useful for small functions that
;;; aren't worth defining a seperate function.
(defmacro key-binding (&rest body)
  (`
    (function (lambda () (interactive) (,@ body )))
  )
)
;;; --------------------------------------------------------------------------
(defun delete-if (test lst)
"Destructively remove all items that satisfy TEST from LIST"
  ;; remove all leading instances
  (while (and lst (listp lst) (funcall test (car lst)))
    (setq lst (cdr lst))
  )
  ;; remove all internal instances - if there is any list left, we know
  ;; that the car should not be removed
  (let
    (
      (prev lst)
    )
    (while (cdr prev)
      (if (funcall test (cadr prev))
	(setcdr prev (cddr prev))
	(setq prev (cdr prev))
      )
    )
    lst					;return value
  )
)
;;;
(defun delete-if-not (test lst)
"Destructively remove all items that do not pass TEST from LIST"
  (delete-if (` (lambda (x) (not ((, test) x)))) lst)
  ;; ^-gnarly, but it works. can't do it straight-forwardly because elisp
  ;; does dynamic scoping, so "test" doesn't get bound correctly and it
  ;; infinitely recurses, so we have to evaluate it here before passing it on.
)
;;;
(defun delete-if-eq (item lst)
"Destructively remove all instances of ITEM from LIST, returning the
modified list"
  (delete-if (function (lambda (x) (eq item x))) lst)
)
;;;
(defun alist-delete (item lst)
"Destructively remove the pair with key ITEM from the ALIST using equal"
  (delete-if (` (lambda (x) (equal (, item) (car x)))) lst)
)
;;; --------------------------------------------------------------------------
(defmacro save-screen-excursion (&rest body)
"Save the current screen and window configuration on that screen,
execute BODY, and then restore the screen and window configuration"
  (let
    (
      (oldscreen (make-symbol "oldscreen"))
      (oldwin (make-symbol "oldwin"))
    )
    (`
      (let
	(
	  ( (, oldscreen) (current-screen))
	  ( (, oldwin) (current-window-configuration))
	)
	(unwind-protect
	  (progn (,@ body))
	  (when (not (null (epoch::make-id (, oldscreen))))
	    (select-screen (, oldscreen))
	    (set-window-configuration (, oldwin))
	    (set-buffer (window-buffer (selected-window)))
	  )
	)
      )
    )
  )
)
;;; --------------------------------------------------------------------------
